/*
 * Author: Kipp Cannon <kipp@gravity.phys.uwm.edu>
 * Revision: $Id: TimeSeries.m4,v 1.1 2005/01/28 23:38:05 kipp Exp $
 */

#ifndef _TIMESERIES_H
#define _TIMESERIES_H

#include <lal/LALDatatypes.h>



/* <lalLaTeX file="TimeSeriesDestroyP">
\idx{XLALDestroyCOMPLEX8TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesDestroyP"> */
void XLALDestroyCOMPLEX8TimeSeries (
	COMPLEX8TimeSeries *series
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesDestroyP">
\idx{LALDestroyCOMPLEX8TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesDestroyP"> */
void LALDestroyCOMPLEX8TimeSeries (
	LALStatus *status,
	COMPLEX8TimeSeries *series
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesCreateP">
\idx{XLALCreateCOMPLEX8TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesCreateP"> */
COMPLEX8TimeSeries *XLALCreateCOMPLEX8TimeSeries (
	const CHAR *name,
	const LIGOTimeGPS *epoch,
	REAL8 f0,
	REAL8 deltaT,
	const LALUnit *sampleUnits,
	size_t length
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesCreateP">
\idx{LALCreateCOMPLEX8TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesCreateP"> */
void LALCreateCOMPLEX8TimeSeries (
	LALStatus *status,
	COMPLEX8TimeSeries **output,
	const CHAR *name,
	LIGOTimeGPS epoch,
	REAL8 f0,
	REAL8 deltaT,
	LALUnit sampleUnits,
	size_t length
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesCutP">
\idx{XLALCutCOMPLEX8TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesCutP"> */
COMPLEX8TimeSeries *XLALCutCOMPLEX8TimeSeries (
	COMPLEX8TimeSeries *series,
	size_t first,
	size_t length
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesCutP">
\idx{LALCutCOMPLEX8TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesCutP"> */
void LALCutCOMPLEX8TimeSeries (
	LALStatus *status,
	COMPLEX8TimeSeries **output,
	COMPLEX8TimeSeries *input,
	size_t first,
	size_t length
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesResizeP">
\idx{XLALResizeCOMPLEX8TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesResizeP"> */
size_t XLALResizeCOMPLEX8TimeSeries (
	COMPLEX8TimeSeries *series,
	int first,
	size_t length
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesResizeP">
\idx{LALResizeCOMPLEX8TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesResizeP"> */
void LALResizeCOMPLEX8TimeSeries (
	LALStatus *status,
	COMPLEX8TimeSeries *series,
	int first,
	size_t length
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesResizeP">
\idx{XLALShrinkCOMPLEX8TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesResizeP"> */
size_t XLALShrinkCOMPLEX8TimeSeries (
	COMPLEX8TimeSeries *series,
	size_t first,
	size_t length
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesResizeP">
\idx{LALShrinkCOMPLEX8TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesResizeP"> */
void LALShrinkCOMPLEX8TimeSeries (
	LALStatus *status,
	COMPLEX8TimeSeries *series,
	size_t first,
	size_t length
);
/* </lalVerbatim> */





/* <lalLaTeX file="TimeSeriesDestroyP">
\idx{XLALDestroyCOMPLEX16TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesDestroyP"> */
void XLALDestroyCOMPLEX16TimeSeries (
	COMPLEX16TimeSeries *series
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesDestroyP">
\idx{LALDestroyCOMPLEX16TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesDestroyP"> */
void LALDestroyCOMPLEX16TimeSeries (
	LALStatus *status,
	COMPLEX16TimeSeries *series
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesCreateP">
\idx{XLALCreateCOMPLEX16TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesCreateP"> */
COMPLEX16TimeSeries *XLALCreateCOMPLEX16TimeSeries (
	const CHAR *name,
	const LIGOTimeGPS *epoch,
	REAL8 f0,
	REAL8 deltaT,
	const LALUnit *sampleUnits,
	size_t length
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesCreateP">
\idx{LALCreateCOMPLEX16TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesCreateP"> */
void LALCreateCOMPLEX16TimeSeries (
	LALStatus *status,
	COMPLEX16TimeSeries **output,
	const CHAR *name,
	LIGOTimeGPS epoch,
	REAL8 f0,
	REAL8 deltaT,
	LALUnit sampleUnits,
	size_t length
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesCutP">
\idx{XLALCutCOMPLEX16TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesCutP"> */
COMPLEX16TimeSeries *XLALCutCOMPLEX16TimeSeries (
	COMPLEX16TimeSeries *series,
	size_t first,
	size_t length
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesCutP">
\idx{LALCutCOMPLEX16TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesCutP"> */
void LALCutCOMPLEX16TimeSeries (
	LALStatus *status,
	COMPLEX16TimeSeries **output,
	COMPLEX16TimeSeries *input,
	size_t first,
	size_t length
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesResizeP">
\idx{XLALResizeCOMPLEX16TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesResizeP"> */
size_t XLALResizeCOMPLEX16TimeSeries (
	COMPLEX16TimeSeries *series,
	int first,
	size_t length
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesResizeP">
\idx{LALResizeCOMPLEX16TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesResizeP"> */
void LALResizeCOMPLEX16TimeSeries (
	LALStatus *status,
	COMPLEX16TimeSeries *series,
	int first,
	size_t length
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesResizeP">
\idx{XLALShrinkCOMPLEX16TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesResizeP"> */
size_t XLALShrinkCOMPLEX16TimeSeries (
	COMPLEX16TimeSeries *series,
	size_t first,
	size_t length
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesResizeP">
\idx{LALShrinkCOMPLEX16TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesResizeP"> */
void LALShrinkCOMPLEX16TimeSeries (
	LALStatus *status,
	COMPLEX16TimeSeries *series,
	size_t first,
	size_t length
);
/* </lalVerbatim> */





/* <lalLaTeX file="TimeSeriesDestroyP">
\idx{XLALDestroyREAL4TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesDestroyP"> */
void XLALDestroyREAL4TimeSeries (
	REAL4TimeSeries *series
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesDestroyP">
\idx{LALDestroyREAL4TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesDestroyP"> */
void LALDestroyREAL4TimeSeries (
	LALStatus *status,
	REAL4TimeSeries *series
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesCreateP">
\idx{XLALCreateREAL4TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesCreateP"> */
REAL4TimeSeries *XLALCreateREAL4TimeSeries (
	const CHAR *name,
	const LIGOTimeGPS *epoch,
	REAL8 f0,
	REAL8 deltaT,
	const LALUnit *sampleUnits,
	size_t length
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesCreateP">
\idx{LALCreateREAL4TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesCreateP"> */
void LALCreateREAL4TimeSeries (
	LALStatus *status,
	REAL4TimeSeries **output,
	const CHAR *name,
	LIGOTimeGPS epoch,
	REAL8 f0,
	REAL8 deltaT,
	LALUnit sampleUnits,
	size_t length
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesCutP">
\idx{XLALCutREAL4TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesCutP"> */
REAL4TimeSeries *XLALCutREAL4TimeSeries (
	REAL4TimeSeries *series,
	size_t first,
	size_t length
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesCutP">
\idx{LALCutREAL4TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesCutP"> */
void LALCutREAL4TimeSeries (
	LALStatus *status,
	REAL4TimeSeries **output,
	REAL4TimeSeries *input,
	size_t first,
	size_t length
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesResizeP">
\idx{XLALResizeREAL4TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesResizeP"> */
size_t XLALResizeREAL4TimeSeries (
	REAL4TimeSeries *series,
	int first,
	size_t length
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesResizeP">
\idx{LALResizeREAL4TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesResizeP"> */
void LALResizeREAL4TimeSeries (
	LALStatus *status,
	REAL4TimeSeries *series,
	int first,
	size_t length
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesResizeP">
\idx{XLALShrinkREAL4TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesResizeP"> */
size_t XLALShrinkREAL4TimeSeries (
	REAL4TimeSeries *series,
	size_t first,
	size_t length
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesResizeP">
\idx{LALShrinkREAL4TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesResizeP"> */
void LALShrinkREAL4TimeSeries (
	LALStatus *status,
	REAL4TimeSeries *series,
	size_t first,
	size_t length
);
/* </lalVerbatim> */





/* <lalLaTeX file="TimeSeriesDestroyP">
\idx{XLALDestroyREAL8TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesDestroyP"> */
void XLALDestroyREAL8TimeSeries (
	REAL8TimeSeries *series
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesDestroyP">
\idx{LALDestroyREAL8TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesDestroyP"> */
void LALDestroyREAL8TimeSeries (
	LALStatus *status,
	REAL8TimeSeries *series
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesCreateP">
\idx{XLALCreateREAL8TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesCreateP"> */
REAL8TimeSeries *XLALCreateREAL8TimeSeries (
	const CHAR *name,
	const LIGOTimeGPS *epoch,
	REAL8 f0,
	REAL8 deltaT,
	const LALUnit *sampleUnits,
	size_t length
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesCreateP">
\idx{LALCreateREAL8TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesCreateP"> */
void LALCreateREAL8TimeSeries (
	LALStatus *status,
	REAL8TimeSeries **output,
	const CHAR *name,
	LIGOTimeGPS epoch,
	REAL8 f0,
	REAL8 deltaT,
	LALUnit sampleUnits,
	size_t length
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesCutP">
\idx{XLALCutREAL8TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesCutP"> */
REAL8TimeSeries *XLALCutREAL8TimeSeries (
	REAL8TimeSeries *series,
	size_t first,
	size_t length
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesCutP">
\idx{LALCutREAL8TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesCutP"> */
void LALCutREAL8TimeSeries (
	LALStatus *status,
	REAL8TimeSeries **output,
	REAL8TimeSeries *input,
	size_t first,
	size_t length
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesResizeP">
\idx{XLALResizeREAL8TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesResizeP"> */
size_t XLALResizeREAL8TimeSeries (
	REAL8TimeSeries *series,
	int first,
	size_t length
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesResizeP">
\idx{LALResizeREAL8TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesResizeP"> */
void LALResizeREAL8TimeSeries (
	LALStatus *status,
	REAL8TimeSeries *series,
	int first,
	size_t length
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesResizeP">
\idx{XLALShrinkREAL8TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesResizeP"> */
size_t XLALShrinkREAL8TimeSeries (
	REAL8TimeSeries *series,
	size_t first,
	size_t length
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesResizeP">
\idx{LALShrinkREAL8TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesResizeP"> */
void LALShrinkREAL8TimeSeries (
	LALStatus *status,
	REAL8TimeSeries *series,
	size_t first,
	size_t length
);
/* </lalVerbatim> */





/* <lalLaTeX file="TimeSeriesDestroyP">
\idx{XLALDestroyINT2TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesDestroyP"> */
void XLALDestroyINT2TimeSeries (
	INT2TimeSeries *series
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesDestroyP">
\idx{LALDestroyINT2TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesDestroyP"> */
void LALDestroyINT2TimeSeries (
	LALStatus *status,
	INT2TimeSeries *series
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesCreateP">
\idx{XLALCreateINT2TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesCreateP"> */
INT2TimeSeries *XLALCreateINT2TimeSeries (
	const CHAR *name,
	const LIGOTimeGPS *epoch,
	REAL8 f0,
	REAL8 deltaT,
	const LALUnit *sampleUnits,
	size_t length
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesCreateP">
\idx{LALCreateINT2TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesCreateP"> */
void LALCreateINT2TimeSeries (
	LALStatus *status,
	INT2TimeSeries **output,
	const CHAR *name,
	LIGOTimeGPS epoch,
	REAL8 f0,
	REAL8 deltaT,
	LALUnit sampleUnits,
	size_t length
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesCutP">
\idx{XLALCutINT2TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesCutP"> */
INT2TimeSeries *XLALCutINT2TimeSeries (
	INT2TimeSeries *series,
	size_t first,
	size_t length
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesCutP">
\idx{LALCutINT2TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesCutP"> */
void LALCutINT2TimeSeries (
	LALStatus *status,
	INT2TimeSeries **output,
	INT2TimeSeries *input,
	size_t first,
	size_t length
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesResizeP">
\idx{XLALResizeINT2TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesResizeP"> */
size_t XLALResizeINT2TimeSeries (
	INT2TimeSeries *series,
	int first,
	size_t length
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesResizeP">
\idx{LALResizeINT2TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesResizeP"> */
void LALResizeINT2TimeSeries (
	LALStatus *status,
	INT2TimeSeries *series,
	int first,
	size_t length
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesResizeP">
\idx{XLALShrinkINT2TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesResizeP"> */
size_t XLALShrinkINT2TimeSeries (
	INT2TimeSeries *series,
	size_t first,
	size_t length
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesResizeP">
\idx{LALShrinkINT2TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesResizeP"> */
void LALShrinkINT2TimeSeries (
	LALStatus *status,
	INT2TimeSeries *series,
	size_t first,
	size_t length
);
/* </lalVerbatim> */





/* <lalLaTeX file="TimeSeriesDestroyP">
\idx{XLALDestroyINT4TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesDestroyP"> */
void XLALDestroyINT4TimeSeries (
	INT4TimeSeries *series
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesDestroyP">
\idx{LALDestroyINT4TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesDestroyP"> */
void LALDestroyINT4TimeSeries (
	LALStatus *status,
	INT4TimeSeries *series
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesCreateP">
\idx{XLALCreateINT4TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesCreateP"> */
INT4TimeSeries *XLALCreateINT4TimeSeries (
	const CHAR *name,
	const LIGOTimeGPS *epoch,
	REAL8 f0,
	REAL8 deltaT,
	const LALUnit *sampleUnits,
	size_t length
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesCreateP">
\idx{LALCreateINT4TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesCreateP"> */
void LALCreateINT4TimeSeries (
	LALStatus *status,
	INT4TimeSeries **output,
	const CHAR *name,
	LIGOTimeGPS epoch,
	REAL8 f0,
	REAL8 deltaT,
	LALUnit sampleUnits,
	size_t length
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesCutP">
\idx{XLALCutINT4TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesCutP"> */
INT4TimeSeries *XLALCutINT4TimeSeries (
	INT4TimeSeries *series,
	size_t first,
	size_t length
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesCutP">
\idx{LALCutINT4TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesCutP"> */
void LALCutINT4TimeSeries (
	LALStatus *status,
	INT4TimeSeries **output,
	INT4TimeSeries *input,
	size_t first,
	size_t length
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesResizeP">
\idx{XLALResizeINT4TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesResizeP"> */
size_t XLALResizeINT4TimeSeries (
	INT4TimeSeries *series,
	int first,
	size_t length
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesResizeP">
\idx{LALResizeINT4TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesResizeP"> */
void LALResizeINT4TimeSeries (
	LALStatus *status,
	INT4TimeSeries *series,
	int first,
	size_t length
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesResizeP">
\idx{XLALShrinkINT4TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesResizeP"> */
size_t XLALShrinkINT4TimeSeries (
	INT4TimeSeries *series,
	size_t first,
	size_t length
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesResizeP">
\idx{LALShrinkINT4TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesResizeP"> */
void LALShrinkINT4TimeSeries (
	LALStatus *status,
	INT4TimeSeries *series,
	size_t first,
	size_t length
);
/* </lalVerbatim> */





/* <lalLaTeX file="TimeSeriesDestroyP">
\idx{XLALDestroyINT8TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesDestroyP"> */
void XLALDestroyINT8TimeSeries (
	INT8TimeSeries *series
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesDestroyP">
\idx{LALDestroyINT8TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesDestroyP"> */
void LALDestroyINT8TimeSeries (
	LALStatus *status,
	INT8TimeSeries *series
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesCreateP">
\idx{XLALCreateINT8TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesCreateP"> */
INT8TimeSeries *XLALCreateINT8TimeSeries (
	const CHAR *name,
	const LIGOTimeGPS *epoch,
	REAL8 f0,
	REAL8 deltaT,
	const LALUnit *sampleUnits,
	size_t length
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesCreateP">
\idx{LALCreateINT8TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesCreateP"> */
void LALCreateINT8TimeSeries (
	LALStatus *status,
	INT8TimeSeries **output,
	const CHAR *name,
	LIGOTimeGPS epoch,
	REAL8 f0,
	REAL8 deltaT,
	LALUnit sampleUnits,
	size_t length
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesCutP">
\idx{XLALCutINT8TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesCutP"> */
INT8TimeSeries *XLALCutINT8TimeSeries (
	INT8TimeSeries *series,
	size_t first,
	size_t length
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesCutP">
\idx{LALCutINT8TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesCutP"> */
void LALCutINT8TimeSeries (
	LALStatus *status,
	INT8TimeSeries **output,
	INT8TimeSeries *input,
	size_t first,
	size_t length
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesResizeP">
\idx{XLALResizeINT8TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesResizeP"> */
size_t XLALResizeINT8TimeSeries (
	INT8TimeSeries *series,
	int first,
	size_t length
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesResizeP">
\idx{LALResizeINT8TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesResizeP"> */
void LALResizeINT8TimeSeries (
	LALStatus *status,
	INT8TimeSeries *series,
	int first,
	size_t length
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesResizeP">
\idx{XLALShrinkINT8TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesResizeP"> */
size_t XLALShrinkINT8TimeSeries (
	INT8TimeSeries *series,
	size_t first,
	size_t length
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesResizeP">
\idx{LALShrinkINT8TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesResizeP"> */
void LALShrinkINT8TimeSeries (
	LALStatus *status,
	INT8TimeSeries *series,
	size_t first,
	size_t length
);
/* </lalVerbatim> */





/* <lalLaTeX file="TimeSeriesDestroyP">
\idx{XLALDestroyUINT2TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesDestroyP"> */
void XLALDestroyUINT2TimeSeries (
	UINT2TimeSeries *series
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesDestroyP">
\idx{LALDestroyUINT2TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesDestroyP"> */
void LALDestroyUINT2TimeSeries (
	LALStatus *status,
	UINT2TimeSeries *series
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesCreateP">
\idx{XLALCreateUINT2TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesCreateP"> */
UINT2TimeSeries *XLALCreateUINT2TimeSeries (
	const CHAR *name,
	const LIGOTimeGPS *epoch,
	REAL8 f0,
	REAL8 deltaT,
	const LALUnit *sampleUnits,
	size_t length
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesCreateP">
\idx{LALCreateUINT2TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesCreateP"> */
void LALCreateUINT2TimeSeries (
	LALStatus *status,
	UINT2TimeSeries **output,
	const CHAR *name,
	LIGOTimeGPS epoch,
	REAL8 f0,
	REAL8 deltaT,
	LALUnit sampleUnits,
	size_t length
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesCutP">
\idx{XLALCutUINT2TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesCutP"> */
UINT2TimeSeries *XLALCutUINT2TimeSeries (
	UINT2TimeSeries *series,
	size_t first,
	size_t length
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesCutP">
\idx{LALCutUINT2TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesCutP"> */
void LALCutUINT2TimeSeries (
	LALStatus *status,
	UINT2TimeSeries **output,
	UINT2TimeSeries *input,
	size_t first,
	size_t length
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesResizeP">
\idx{XLALResizeUINT2TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesResizeP"> */
size_t XLALResizeUINT2TimeSeries (
	UINT2TimeSeries *series,
	int first,
	size_t length
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesResizeP">
\idx{LALResizeUINT2TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesResizeP"> */
void LALResizeUINT2TimeSeries (
	LALStatus *status,
	UINT2TimeSeries *series,
	int first,
	size_t length
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesResizeP">
\idx{XLALShrinkUINT2TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesResizeP"> */
size_t XLALShrinkUINT2TimeSeries (
	UINT2TimeSeries *series,
	size_t first,
	size_t length
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesResizeP">
\idx{LALShrinkUINT2TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesResizeP"> */
void LALShrinkUINT2TimeSeries (
	LALStatus *status,
	UINT2TimeSeries *series,
	size_t first,
	size_t length
);
/* </lalVerbatim> */





/* <lalLaTeX file="TimeSeriesDestroyP">
\idx{XLALDestroyUINT4TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesDestroyP"> */
void XLALDestroyUINT4TimeSeries (
	UINT4TimeSeries *series
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesDestroyP">
\idx{LALDestroyUINT4TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesDestroyP"> */
void LALDestroyUINT4TimeSeries (
	LALStatus *status,
	UINT4TimeSeries *series
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesCreateP">
\idx{XLALCreateUINT4TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesCreateP"> */
UINT4TimeSeries *XLALCreateUINT4TimeSeries (
	const CHAR *name,
	const LIGOTimeGPS *epoch,
	REAL8 f0,
	REAL8 deltaT,
	const LALUnit *sampleUnits,
	size_t length
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesCreateP">
\idx{LALCreateUINT4TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesCreateP"> */
void LALCreateUINT4TimeSeries (
	LALStatus *status,
	UINT4TimeSeries **output,
	const CHAR *name,
	LIGOTimeGPS epoch,
	REAL8 f0,
	REAL8 deltaT,
	LALUnit sampleUnits,
	size_t length
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesCutP">
\idx{XLALCutUINT4TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesCutP"> */
UINT4TimeSeries *XLALCutUINT4TimeSeries (
	UINT4TimeSeries *series,
	size_t first,
	size_t length
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesCutP">
\idx{LALCutUINT4TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesCutP"> */
void LALCutUINT4TimeSeries (
	LALStatus *status,
	UINT4TimeSeries **output,
	UINT4TimeSeries *input,
	size_t first,
	size_t length
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesResizeP">
\idx{XLALResizeUINT4TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesResizeP"> */
size_t XLALResizeUINT4TimeSeries (
	UINT4TimeSeries *series,
	int first,
	size_t length
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesResizeP">
\idx{LALResizeUINT4TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesResizeP"> */
void LALResizeUINT4TimeSeries (
	LALStatus *status,
	UINT4TimeSeries *series,
	int first,
	size_t length
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesResizeP">
\idx{XLALShrinkUINT4TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesResizeP"> */
size_t XLALShrinkUINT4TimeSeries (
	UINT4TimeSeries *series,
	size_t first,
	size_t length
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesResizeP">
\idx{LALShrinkUINT4TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesResizeP"> */
void LALShrinkUINT4TimeSeries (
	LALStatus *status,
	UINT4TimeSeries *series,
	size_t first,
	size_t length
);
/* </lalVerbatim> */





/* <lalLaTeX file="TimeSeriesDestroyP">
\idx{XLALDestroyUINT8TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesDestroyP"> */
void XLALDestroyUINT8TimeSeries (
	UINT8TimeSeries *series
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesDestroyP">
\idx{LALDestroyUINT8TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesDestroyP"> */
void LALDestroyUINT8TimeSeries (
	LALStatus *status,
	UINT8TimeSeries *series
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesCreateP">
\idx{XLALCreateUINT8TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesCreateP"> */
UINT8TimeSeries *XLALCreateUINT8TimeSeries (
	const CHAR *name,
	const LIGOTimeGPS *epoch,
	REAL8 f0,
	REAL8 deltaT,
	const LALUnit *sampleUnits,
	size_t length
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesCreateP">
\idx{LALCreateUINT8TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesCreateP"> */
void LALCreateUINT8TimeSeries (
	LALStatus *status,
	UINT8TimeSeries **output,
	const CHAR *name,
	LIGOTimeGPS epoch,
	REAL8 f0,
	REAL8 deltaT,
	LALUnit sampleUnits,
	size_t length
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesCutP">
\idx{XLALCutUINT8TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesCutP"> */
UINT8TimeSeries *XLALCutUINT8TimeSeries (
	UINT8TimeSeries *series,
	size_t first,
	size_t length
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesCutP">
\idx{LALCutUINT8TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesCutP"> */
void LALCutUINT8TimeSeries (
	LALStatus *status,
	UINT8TimeSeries **output,
	UINT8TimeSeries *input,
	size_t first,
	size_t length
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesResizeP">
\idx{XLALResizeUINT8TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesResizeP"> */
size_t XLALResizeUINT8TimeSeries (
	UINT8TimeSeries *series,
	int first,
	size_t length
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesResizeP">
\idx{LALResizeUINT8TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesResizeP"> */
void LALResizeUINT8TimeSeries (
	LALStatus *status,
	UINT8TimeSeries *series,
	int first,
	size_t length
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesResizeP">
\idx{XLALShrinkUINT8TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesResizeP"> */
size_t XLALShrinkUINT8TimeSeries (
	UINT8TimeSeries *series,
	size_t first,
	size_t length
);
/* </lalVerbatim> */

/* <lalLaTeX file="TimeSeriesResizeP">
\idx{LALShrinkUINT8TimeSeries ()}
</lalLaTeX> <lalVerbatim file="TimeSeriesResizeP"> */
void LALShrinkUINT8TimeSeries (
	LALStatus *status,
	UINT8TimeSeries *series,
	size_t first,
	size_t length
);
/* </lalVerbatim> */



#endif  /* _TIMESERIES_H */
